@@ -54,9 +54,19 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <unistd.h>
+
+#ifdef _WIN32
+/* This is for Windows Vista and later, for inet_pton() */
+# define _WIN32_WINNT 0x0600
+
+# include <winsock2.h>
+# include <ws2tcpip.h>
+#else
+# include <sys/socket.h>
+# include <netinet/in.h>
+# include <unistd.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -64,8 +74,12 @@
 #include <time.h>
 #include <errno.h>
 #include <limits.h> /* for UINT_MAX */
-#include <langinfo.h>
-#include <arpa/inet.h>
+
+#ifndef _WIN32
+# include <langinfo.h>
+# include <arpa/inet.h>
+#endif
+
 #ifdef HAVE_ALLOCA_H
 #	include <alloca.h>
 #endif
@@ -88,30 +102,12 @@
 #	include <unistr.h>
 #endif
 
-#include <libpsl.h>
-
-#ifndef HAVE_STRNDUP
-/* I found no strndup on my old SUSE 7.3 test system (gcc 2.95) */
-
-static char *strndup(const char *s, size_t n)
-{
-	char *dst;
-	size_t s_len = strlen(s);
-
-	if (s_len > n)
-		n = s_len;
-
-	dst = malloc(n + 1);
-
-	if (dst) {
-		memcpy(dst, s, n);
-		dst[n] = 0;
-	}
-
-	return dst;
-}
+#ifndef WINICONV_CONST
+#  define WINICONV_CONST
 #endif
 
+#include <libpsl.h>
+
 /**
  * SECTION:libpsl
  * @short_description: Public Suffix List library functions
@@ -170,7 +166,17 @@ struct _psl_ctx_st {
 };
 
 /* include the PSL data generated by psl-make-dafsa */
+#if defined(BUILTIN_GENERATOR_LIBICU) || defined(BUILTIN_GENERATOR_LIBIDN2) || defined(BUILTIN_GENERATOR_LIBIDN)
 #include "suffixes_dafsa.c"
+#else
+static const unsigned char kDafsa[] = "";
+static time_t _psl_file_time = 0;
+static int _psl_nsuffixes = 0;
+static int _psl_nexceptions = 0;
+static int _psl_nwildcards = 0;
+static const char _psl_sha1_checksum[] = "";
+static const char _psl_filename[] = "";
+#endif
 
 /* references to these PSLs will result in lookups to built-in data */
 static const psl_ctx_t
@@ -1051,6 +1057,9 @@ int psl_is_public_suffix2(const psl_ctx_t *psl, const char *domain, int type)
  */
 const char *psl_unregistrable_domain(const psl_ctx_t *psl, const char *domain)
 {
+	int nlabels = 0;
+	const char *p;
+
 	if (!psl || !domain)
 		return NULL;
 
@@ -1058,9 +1067,6 @@ const char *psl_unregistrable_domain(const psl_ctx_t *psl, const char *domain)
 	 * In the main loop we introduce a O(N^2) behavior to avoid code duplication.
 	 * To avoid nasty CPU hogging, we limit the lookup to max. 8 domain labels to the right.
 	 */
-
-	int nlabels = 0;
-	const char *p;
 	for (p = domain + strlen(domain) - 1; p >= domain; p--) {
 		if (*p == '.' && ++nlabels > 8) {
 			domain = p + 1;
@@ -1106,6 +1112,7 @@ const char *psl_unregistrable_domain(const psl_ctx_t *psl, const char *domain)
 const char *psl_registrable_domain(const psl_ctx_t *psl, const char *domain)
 {
 	const char *p, *regdom = NULL;
+	int nlabels = 0;
 
 	if (!psl || !domain || *domain == '.')
 		return NULL;
@@ -1114,8 +1121,6 @@ const char *psl_registrable_domain(const psl_ctx_t *psl, const char *domain)
 	 * In the main loop we introduce a O(N^2) behavior to avoid code duplication.
 	 * To avoid nasty CPU hogging, we limit the lookup to max. 8 domain labels to the right.
 	 */
-
-	int nlabels = 0;
 	for (p = domain + strlen(domain) - 1; p >= domain; p--) {
 		if (*p == '.' && ++nlabels > 8) {
 			domain = p + 1;
@@ -1789,8 +1794,13 @@ out:
 	do {
 		/* find out local charset encoding */
 		if (!encoding) {
+#ifdef HAVE_NL_LANGINFO
 			encoding = nl_langinfo(CODESET);
-
+#elif defined _WIN32
+			static char buf[16];
+			snprintf(buf, sizeof(buf), "CP%u", GetACP());
+			encoding = buf;
+#endif
 			if (!encoding || !*encoding)
 				encoding = "ASCII";
 		}
@@ -1808,7 +1818,7 @@ out:
 				if (!dst) {
 					ret = PSL_ERR_NO_MEM;
 				}
-				else if (iconv(cd, &tmp, &tmp_len, &dst_tmp, &dst_len_tmp) != (size_t)-1
+				else if (iconv(cd, (WINICONV_CONST char **)&tmp, &tmp_len, &dst_tmp, &dst_len_tmp) != (size_t)-1
 					&& iconv(cd, NULL, NULL, &dst_tmp, &dst_len_tmp) != (size_t)-1)
 				{
 					/* start size for u8_tolower internal memory allocation.
